import 'dart:async';

import 'package:async_tools/async_tools.dart';
import 'package:veilid_support/veilid_support.dart';

import '../../proto/proto.dart' as proto;
import '../account_manager.dart';

typedef AccountRecordState = proto.Account;
typedef _SspUpdateState = (
  AccountSpec accountSpec,
  Future<void> Function() onSuccess
);

/// The saved state of a VeilidChat Account on the DHT
/// Used to synchronize status, profile, and options for a specific account
/// across multiple clients. This DHT record is the 'source of truth' for an
/// account and is privately encrypted with an owned record from the 'userLogin'
/// tabledb-local storage, encrypted by the unlock code for the account.
class AccountRecordCubit extends DefaultDHTRecordCubit<AccountRecordState> {
  AccountRecordCubit(
      {required LocalAccount localAccount, required UserLogin userLogin})
      : super(
            decodeState: proto.Account.fromBuffer,
            open: () => _open(localAccount, userLogin));

  static Future<DHTRecord> _open(
      LocalAccount localAccount, UserLogin userLogin) async {
    // Record not yet open, do it
    final pool = DHTRecordPool.instance;
    final record = await pool.openRecordOwned(
        userLogin.accountRecordInfo.accountRecord,
        debugName: 'AccountRecordCubit::_open::AccountRecord',
        parent: localAccount.superIdentity.currentInstance.recordKey);

    return record;
  }

  @override
  Future<void> close() async {
    await _sspUpdate.close();
    await super.close();
  }

  ////////////////////////////////////////////////////////////////////////////
  // Public Interface

  void updateAccount(
      AccountSpec accountSpec, Future<void> Function() onChanged) {
    _sspUpdate.updateState((accountSpec, onChanged), (state) async {
      await _updateAccountAsync(state.$1, state.$2);
    });
  }

  Future<void> _updateAccountAsync(
      AccountSpec accountSpec, Future<void> Function() onChanged) async {
    var changed = true;
    await record?.eventualUpdateProtobuf(proto.Account.fromBuffer, (old) async {
      if (old == null) {
        return null;
      }

      final oldAccountSpec = AccountSpec.fromProto(old);
      changed = oldAccountSpec != accountSpec;
      if (!changed) {
        return null;
      }

      return accountSpec.updateProto(old);
    });
    if (changed) {
      await onChanged();
    }
  }

  final _sspUpdate = SingleStateProcessor<_SspUpdateState>();
}
